require("sound.lua")

function love.run()
  _r = math.random
  _f = math.floor
  _c = math.cos
  _s = math.sin
  _sq = math.sqrt
  _at2 = math.atan2
  _d2r = math.rad
  _r2d = math.deg
  _m = math.mod
  pi = math.pi
  gr = love.graphics
  ti = love.timer
  ms = love.mouse
  kb = love.keyboard
  math.randomseed(os.time())

	love.load(arg)
	local dt = 0
	while true do
	  dt = ti.getDelta()
	  time = ti.getTime()
		ti.step()
		gr.clear()
		love.update(dt)
		love.draw(dt)
		if love.event then
			for e,a,b,c in love.event.poll() do
				if e == "q" then
					if love.audio then love.audio.stop() end
				return
				end
				love.handlers[e](a,b,c)
			end
		end
		ti.sleep(1)
		gr.present()
	end
end

function love.load()
  screenW = gr.getWidth()
  screenH = gr.getHeight()

	--muzak = {}
	--muzak["bgm"] = love.audio.newSource("audio/bgm.ogg", "stream")

	sfx = {}
	sfx["player_fire_1"] = love.sound.newSoundData("audio/97822__phisto99__FS_021_Laser_or_shoot.wav")
	sfx["player_fire_5"] = love.sound.newSoundData("audio/97820__phisto99__FS_019_Laser_or_shoot.wav")
	sfx["enemy_fire_1"] = love.sound.newSoundData("audio/97821__phisto99__FS_020_Laser_or_shoot.wav")

  water, lava, slime = 11, 12, 13
  dirt, grass, stone, none = 1, 2, 3, 0

	level = { offset = 0, speed = 64, scale = 1, map = {}, liquid = water, liquidHeight = 56, scrollPause = .5 }
  player = {
    x = 50, y = 250, tileX = 0, xs = 256, ys = 256,
    energy = 100, energyMax = 100, hasShield = false, shieldTime = 0, shieldRad = 0,
    lives = 0, kills = 0, timeShot = 0, timeBetween = .05, multiplier = 1, multiplierCounter = 0, score = 0
  }

  color = {}
  color[dirt] = { {95/2,56/2,21/2}, {95,56,21} }
  color[grass] = { {0,255/2,0}, {0,255,0} }

  color[water] = { {0,0,255/2}, {0,0,255} }
  color[lava] = { {255,255/2,0}, {255,255,0} }
  color[slime] = { {0,255/2,0}, {0,255,0} }

	for x=0,1000-1 do
	  level.map[x] = {}
	  for y=0,15-1 do
	    local e
	    if _r(1,20) == 1 and x > 24 and y > 2 and y < 13 then e = _r(1,2) else e = 0 end
      level.map[x][y] = { tile = 0, enemy = e }
	  end
	  level.map[x].ceil = _r(24,40)
	  level.map[x].ceilType = dirt
	  level.map[x].floor = 40+_r(24,40)
	  level.map[x].floorType = grass
	  level.map[x].enemies = {}
    if _r(0,9) == 0 then
  	  --level.map[x].fDeco = _r(1,2)
  	end
	end

  local wl = _r(10,20)
  local wl2 = wl+_r(25,35)
  for i=wl,wl2 do
    level.map[i].floor = 0
  end

  local j = 1
  for i=wl-4,wl do
    level.map[i].floor = level.map[i].floor / j
    j = j + 1
  end

  local j = 5
  for i=wl2,wl2+4 do
    level.map[i].floor = level.map[i].floor / j
    j = j - 1
  end


  level.scale = (screenW/800)

  enemy = {}
  bullet = {}
  explosion = {}

  --bgm = love.audio.newSource("bgm.ogg", "stream")
  --love.audio.play(bgm)

	fsmall = gr.newFont(love._vera_ttf, 12*level.scale)
	fbig = gr.newFont(love._vera_ttf, 20*level.scale)

  gr.setBackgroundColor(0,0,0)

  loadAlphabet()

  paused = false

  lw = math.max(1*level.scale,1)
  pauseColor = {255,255,255}
end

function love.update(dt)
  local kLeft = kb.isDown("left")
  local kRight = kb.isDown("right")
  local kUp = kb.isDown("up")
  local kDown = kb.isDown("down")
  local kSpace = kb.isDown(" ")

  if paused then

  else
    --level.speed = level.speed + 10*dt
    player.tileX = _f(player.x/32)
    level.scrollPause = level.scrollPause - dt
    if level.scrollPause <= 0 then
      level.offset = level.offset + level.speed * dt
      player.x = player.x + level.speed * dt
    end

    if player.hasShield then
      player.shieldTime = player.shieldTime - dt
      if player.shieldTime <= 0 then player.hasShield = false end
      if player.shieldRad < 48 then player.shieldRad = player.shieldRad + 256*dt if player.shieldRad > 48 then player.shieldRad = 48 end end
    else
      if player.shieldRad > 0 then player.shieldRad = player.shieldRad - 256*dt if player.shieldRad < 0 then player.shieldRad = 0 end end
    end

    player.timeShot = player.timeShot - dt
    if player.timeShot < 0 then player.timeShot = 0 end

    if kSpace then
      if player.timeShot <= 0 then
        spawnBullet(player.x, player.y, 100, 0, 768, "player")
        playSound("player_fire_1")
        player.timeShot = player.timeBetween
      end
    end

    if kLeft then
      player.x = player.x - player.xs * dt
      if player.x < level.offset + 20 then player.x = level.offset + 20 end
    elseif kRight then
      player.x = player.x + player.xs * dt
      if player.x > level.offset + 780 then player.x = level.offset + 780 end
    end

    if kUp then
      player.y = player.y - player.ys * dt
    elseif kDown then
      player.y = player.y + player.ys * dt
    end

    for i, e in pairs(enemy) do
      if e.dead == false then
        local s = 96

        e.tsb = e.tsb - dt
        if e.tsb < 0 and e.x < level.offset+750 then
          spawnBullet(e.x, e.y, 5, 180, 512, "enemy")
          playSound("enemy_fire_1")
          e.tsb = 1
        end

        if e.type == 2 then s = 128 end
        e.x = e.x - s * dt
        if e.type == 2 then
          e.y = e.y + _s(ti.getTime() * 2)
        end

        if e.x < level.offset - 32 then
          e.dead = true
        end
        for j, b in pairs(bullet) do
          if b.x > level.offset + 820 or b.x < level.offset - 20 then
            b.dead = true
          end
          if b.who == "player" then
            if overlap(e.x-16,e.y-16,32,32,b.x-32,b.y-3,32,6) and e.dead == false and b.dead == false then
              if b.type == 1 then b.dead = true end
              e.dead = true
              spawnExplosion(e.x,e.y)
              player.score = player.score + e.points * player.multiplier
              player.multiplierCounter = player.multiplierCounter + 1
              if player.multiplierCounter >= 10 then
                player.multiplierCounter = 0
                if player.multiplier < 10 then player.multiplier = player.multiplier + 1 end
              end
              player.kills = player.kills + 1
            end
          end
        end
      end
    end

    for i, b in pairs(bullet) do
      if b.who == "enemy" then
        if overlap(player.x-16,player.y-16,32,32,b.x-32,b.y-3,32,6) and b.dead == false then
          b.dead = true
          spawnExplosion(player.x,player.y)
          damagePlayer(10)
        end
      end

      b.life = b.life - dt
      b.x = b.x + (_c(b.angle * pi / 180) * (b.speed * dt)) * 1
      b.y = b.y + (_s(b.angle * pi / 180) * (b.speed * dt)) * 1
      if b.life < 0 then b.dead = true end
    end

    if player.energy <= 0 then
      killPlayer()
    end

    updateExplosions(dt)
  end
end

function love.draw()
  gr.setLineWidth(lw)
  --gr.setBackgroundColor(0,0,0)
	local fps = love.timer.getFPS()

  local xlb = _f((level.offset)/32)
  local xub = xlb + 26
  if xlb < 0 then xlb = 0 end
  if xlb ~= oxlb then
    loadEnemies(xub)

    for i=#enemy,1,-1 do
      if enemy[i].dead == true then
        table.remove(enemy, i)
      end
    end

    for i=#explosion,1,-1 do
      if explosion[i].dead == true then
        table.remove(explosion, i)
      end
    end

    for i=#bullet,1,-1 do
      if bullet[i].dead == true then
        table.remove(bullet, i)
      end
    end
  end


	for i, b in pairs(bullet) do
    if b.dead == false then
      gr.setColor(255,0,0)
      gr.circle("line", (b.x-level.offset)*level.scale, b.y*level.scale, 3*level.scale)
    end
	end

	for i, e in pairs(enemy) do
    if e.dead == false then
      if e.type == 1 then
        gr.setColor(255,0,0)
        gr.circle("line", (e.x-level.offset)*level.scale, e.y*level.scale, 16*level.scale, 20)
      elseif e.type == 2 then
        gr.setColor(0,0,255)
        gr.circle("line", (e.x-level.offset)*level.scale, e.y*level.scale, 16*level.scale, 20)
      end
    end
	end

	gr.setColor(0,255,0)
	gr.circle("line", (player.x-level.offset)*level.scale, player.y*level.scale, 16*level.scale, 20)

	if player.shieldRad > 0 then
	  gr.setColor(0,0,255)
	  gr.circle("line", (player.x-level.offset)*level.scale, player.y*level.scale, player.shieldRad+(_s(time*5)*3), 100)
	end

  if level.liquid > 0 then
    local bob = (500-level.liquidHeight) + (4 * _s(time))
    local bob2 = (500-level.liquidHeight) + (4 * _c(time))
    gr.setColor(unpack(color[level.liquid][1]))
    gr.polygon("fill",
      0, bob*level.scale,
      800*level.scale, bob2*level.scale,
      800*level.scale, 500*level.scale,
      0, 500*level.scale
    )
    gr.setColor(unpack(color[level.liquid][2]))
    gr.line(0,bob*level.scale, 800*level.scale,bob2*level.scale)
  end

	for x=xlb,xub do
	  if x > 0 then
      if level.map[x].fDeco then
        gr.circle("fill", ((x*32)-level.offset)*level.scale+16, level.map[x-1].floor*level.scale+.5-10, 10)
        gr.setColor(95/2,56/2,21/2)
        gr.rectangle("fill", (
          (x*32)-level.offset)*level.scale+16, level.map[x-1].floor*level.scale+.5-10, 4, 10
        )
      end

      --CEILING
      gr.setColor(unpack(color[level.map[x].ceilType][1]))
      gr.polygon("fill",
	      (((x-1)*32)-level.offset)*level.scale+.5, 0,
	      ((x*32)-level.offset)*level.scale+.5, 0,
	      ((x*32)-level.offset)*level.scale+.5, level.map[x].ceil*level.scale+.5,
	      (((x-1)*32)-level.offset)*level.scale+.5, level.map[x-1].ceil*level.scale+.5
	    )

      gr.setColor(unpack(color[level.map[x].ceilType][2]))
	    gr.line(
	      (((x-1)*32)-level.offset)*level.scale+.5,
	      level.map[x-1].ceil*level.scale+.5,
	      ((x*32)-level.offset)*level.scale+.5,
	      level.map[x].ceil*level.scale+.5
	    )

      if level.map[x-1].floor > 0 or level.map[x].floor > 0 then
        --FLOOR
        gr.setColor(unpack(color[level.map[x].floorType][1]))
        gr.polygon("fill",
          (((x-1)*32)-level.offset)*level.scale+.5, 500*level.scale,
          ((x*32)-level.offset)*level.scale+.5, 500*level.scale,
          ((x*32)-level.offset)*level.scale+.5, (500-level.map[x].floor)*level.scale+.5,
          (((x-1)*32)-level.offset)*level.scale+.5, (500-level.map[x-1].floor)*level.scale+.5
        )

        gr.setColor(unpack(color[level.map[x].floorType][2]))
        gr.line(
          (((x-1)*32)-level.offset)*level.scale+.5,
          (500-level.map[x-1].floor)*level.scale+.5,
          ((x*32)-level.offset)*level.scale+.5,
          (500-level.map[x].floor)*level.scale+.5
        )
      end
	  end
	end


  if player.y-20 < level.map[player.tileX].ceil then
    drawWord("DDD",10,10,16,20,{255,255,255})
  end
  if player.y+20 > 500-level.map[player.tileX].floor then
    drawWord("CCC",10,10,16,20,{255,255,255})
  end

  drawWord("FPS:"..fps,10,10,8,12,{255,255,255})
  drawWord("BULLETS:"..#bullet,10,30,8,12,{255,255,255})
  drawWord("ENEMIES:"..#enemy,10,50,8,12,{255,255,255})

  drawWord("KILLS:"..player.kills,500,screenH/level.scale-30,16,20,{255,255,255})

  if player.hasShield then
    gr.setColor(255,255/2,0,255/2)
  else
    gr.setColor(255,255,255,255/2)
  end
  gr.rectangle("fill", 680*level.scale, (screenH/level.scale-30)*level.scale, player.energy*level.scale, 20*level.scale)
  gr.setColor(255,255,255)
  gr.rectangle("line", 680*level.scale, (screenH/level.scale-30)*level.scale, 100*level.scale, 20*level.scale)

  drawWord("x"..player.multiplier,160,screenH/level.scale-30,16,20,{255,255,255})
  drawWord(formatNumber(player.score),260,screenH/level.scale-30,16,20,{255,255,255})

  --drawWord(player.tileX,200,470,16,20,{255,255,255})

  oxlb = xlb

  drawExplosions()

  local s = 1+_s(time)
  --[[drawWord("ABCDEFGHIJKLM",40,50,32,48,{255,255,255})
  drawWord("NOPQRSTUVWXYZ",40,110,32,48,{255,0,255})
  drawWord("!.,:$",40,170,32,48,{255,255,0})
  drawWord("1234567890",40,230,32,48,{0,255,255})--]]

  if paused then
    doPauseColor()
    drawWord("PAUSED", 295,100,32,40,pauseColor)
  end
end

function doPauseColor()
  pauseColor[1] = _r(100,255)
  pauseColor[2] = _r(100,255)
  pauseColor[3] = _r(100,255)
end

function killPlayer()
  player.lives = player.lives - 1
  player.x = level.offset + 40
  player.y = 250
  level.scrollPause = 3
  player.energy = player.energyMax
  player.multiplier = 1
end

function giveSheild()
  player.hasShield = true
  player.shieldTime = 10
end

function damagePlayer(d)
  if player.hasShield then
    d = d / 2
  end
  player.energy = player.energy - d
end

function loadAlphabet()
  letter = {}
  letter["A"] = {
    { { 0,1 }, { .5,0 } },
    { { .5,0 }, { 1,1 } },
    { { .25,.5 }, { .75,.5 } }
  }
  letter["B"] = {
    { { 0,0 }, { 0,1 } },
    { { 0,0 }, { .75,0 } },
    { { 0,.5 }, { 1,.5 } },
    { { 0,1 }, { 1,1 } },
    { { .75,0 }, { .75,.5 } },
    { { 1,.5 }, { 1,1 } }
  }
  letter["C"] = {
    { { 0,0 }, { 1,0 } },
    { { 0,0 }, { 0,1 } },
    { { 0,1 }, { 1,1 } }
  }
  letter["D"] = {
    { { 0,0 }, { 0,1 } },
    { { 0,0 }, { .75,0 } },
    { { .75,0 }, { 1,.25 } },
    { { 1,.25 }, { 1,.75 } },
    { { 1,.75 }, { .75,1 } },
    { { .75,1 }, { 0,1 } }
  }
  letter["E"] = {
    { { 0,0 }, { 1,0 } },
    { { 0,0 }, { 0,1 } },
    { { 0,1 }, { 1,1 } },
    { { 0,.5 }, { .75,.5 } }
  }
  letter["F"] = {
    { { 0,0 }, { 1,0 } },
    { { 0,0 }, { 0,1 } },
    { { 0,.5 }, { .75,.5 } }
  }
  letter["G"] = {
    { { 0,0 }, { 1,0 } },
    { { 0,0 }, { 0,1 } },
    { { 0,1 }, { 1,1 } },
    { { 1,1 }, { 1,.5 } },
    { { 1,.5 }, { .5,.5 } }
  }
  letter["H"] = {
    { { 0,1 }, { 0,0 } },
    { { 1,0 }, { 1,1 } },
    { { 0,.5 }, { 1,.5 } }
  }
  letter["I"] = {
    { { .5,0 }, { .5,1 } }
  }
  letter["J"] = {
    { { 1,0 }, { 1,1 } },
    { { 1,1 }, { 0,1 } },
    { { 0,1 }, { 0,.75 } }
  }
  letter["K"] = {
    { { 0,0 }, { 0,1 } },
    { { 0,.5 }, { 1,0 } },
    { { 0,.5 }, { 1,1 } }
  }
  letter["L"] = {
    { { 0,0 }, { 0,1 } },
    { { 0,1 }, { 1,1 } }
  }
  letter["M"] = {
    { { 0,0 }, { 0,1 } },
    { { 0,0 }, { .5,1 } },
    { { .5,1 }, { 1,0 } },
    { { 1,1 }, { 1,0 } }
  }
  letter["N"] = {
    { { 0,0 }, { 0,1 } },
    { { 0,0 }, { 1,1 } },
    { { 1,1 }, { 1,0 } }
  }
  letter["O"] = {
    { { 0,0 }, { 1,0 } },
    { { 0,0 }, { 0,1 } },
    { { 0,1 }, { 1,1 } },
    { { 1,0 }, { 1,1 } }
  }
  letter["P"] = {
    { { 0,0 }, { 1,0 } },
    { { 0,0 }, { 0,1 } },
    { { 0,.5 }, { 1,.5 } },
    { { 1,0 }, { 1,.5 } }
  }
  letter["Q"] = {
    { { 0,0 }, { 1,0 } },
    { { 0,0 }, { 0,1 } },
    { { 0,1 }, { 1,1 } },
    { { 1,0 }, { 1,1 } },
    { { .75,.75 }, { 1.125,1.125 } }
  }
  letter["R"] = {
    { { 0,0 }, { 1,0 } },
    { { 0,0 }, { 0,1 } },
    { { 0,.5 }, { 1,.5 } },
    { { 1,0 }, { 1,.5 } },
    { { .5,.5 }, { 1,1 } }
  }
  letter["S"] = {
    { { 1,0 }, { 0,0 } },
    { { 0,0 }, { 0,.5 } },
    { { 0,.5 }, { 1,.5 } },
    { { 1,.5 }, { 1,1 } },
    { { 1,1 }, { 0,1 } }
  }
  letter["T"] = {
    { { .5,0 }, { .5,1 } },
    { { 0,0 }, { 1,0 } }
  }
  letter["U"] = {
    { { 0,0 }, { 0,1 } },
    { { 0,1 }, { 1,1 } },
    { { 1,1 }, { 1,0 } }
  }
  letter["V"] = {
    { { 0,0 }, { .5,1 } },
    { { .5,1 }, { 1,0 } }
  }
  letter["W"] = {
    { { 0,0 }, { .25,1 } },
    { { .25,1 }, { .5,0 } },
    { { .5,0 }, { .75,1 } },
    { { .75,1 }, { 1,0 } }
  }
  letter["X"] = {
    { { 0,0 }, { 1,1 } },
    { { 1,0 }, { 0,1 } }
  }
  letter["Y"] = {
    { { 0,0 }, { .5,.5 } },
    { { .5,.5 }, { 1,0 } },
    { { .5,.5 }, { .5,1 } }
  }
  letter["Z"] = {
    { { 0,0 }, { 1,0 } },
    { { 1,0 }, { 0,1 } },
    { { 0,1 }, { 1,1 } }
  }

  letter["x"] = {
    { { 0,.25 }, { 1,1 } },
    { { 1,.25 }, { 0,1 } }
  }

  letter["!"] = {
    { { .5,0 }, { .5,.7 } },
    { { .5,.85 }, { .5,1 } }
  }
  letter["."] = {
    { { .5,.85 }, { .5,1 } }
  }
  letter[":"] = {
    { { .15,.85 }, { .15,1 } },
    { { .15,0 }, { .15,.15 } }
  }
  letter[","] = {
    { { .5,.85 }, { .5,1 } },
    { { .5,1 }, { .35,1.12 } }
  }
  letter["$"] = {
    { { 1,0 }, { 0,0 } },
    { { 0,0 }, { 0,.5 } },
    { { 0,.5 }, { 1,.5 } },
    { { 1,.5 }, { 1,1 } },
    { { 1,1 }, { 0,1 } },
    { { .5,-.2 }, { .5,1.2 } }
  }

  letter["0"] = {
    { { 0,0 }, { 1,0 } },
    { { 1,0 }, { 1,1 } },
    { { 1,1 }, { 0,1 } },
    { { 0,1 }, { 0,0 } },
    { { 1,0 }, { 0,1 } }
  }
  letter["1"] = {
    { { .5,0 }, { .5,1 } },
    { { .5,0 }, { .25,.25 } },
    { { .25,1 }, { .75,1 } }
  }
  letter["2"] = {
    { { 0,1 }, { 1,1 } },
    { { 0,1 }, { 0,.75 } },
    { { 0,.75 }, { 1,.25 } },
    { { 1,.25 }, { 1,0 } },
    { { 1,0 }, { 0,0 } },
    { { 0,0 }, { 0,.25 } }
  }
  letter["3"] = {
    { { 0,0 }, { 1,0 } },
    { { .25,.5 }, { 1,.5 } },
    { { 0,1 }, { 1,1 } },
    { { 1,0 }, { 1,1 } }
  }
  letter["4"] = {
    { { .75,0 }, { .75,1 } },
    { { 0,.5 }, { 1,.5 } },
    { { .125,0 }, { 0,.5 } }
  }
  letter["5"] = {
    { { 1,0 }, { 0,0 } },
    { { 0,0 }, { 0,.5 } },
    { { 0,.5 }, { 1,.5 } },
    { { 1,.5 }, { 1,1 } },
    { { 1,1 }, { 0,1 } }
  }
  letter["6"] = {
    { { 1,0 }, { 0,0 } },
    { { 0,0 }, { 0,1 } },
    { { 0,.5 }, { 1,.5 } },
    { { 1,.5 }, { 1,1 } },
    { { 1,1 }, { 0,1 } }
  }
  letter["7"] = {
    { { 0,0 }, { 1,0 } },
    { { 1,0 }, { .5,1 } }
  }
  letter["8"] = {
    { { 1,0 }, { 0,0 } },
    { { 0,0 }, { 0,1 } },
    { { 0,.5 }, { 1,.5 } },
    { { 1,0 }, { 1,1 } },
    { { 1,1 }, { 0,1 } }
  }
  letter["9"] = {
    { { 1,0 }, { 0,0 } },
    { { 0,0 }, { 0,.5 } },
    { { 0,.5 }, { 1,.5 } },
    { { 1,0 }, { 1,1 } },
    { { 1,1 }, { 0,1 } }
  }
end

function drawWord(t,x,y,sx,sy,c)
  t = tostring(t)
  local l
  local xx = 0
  local spa = (sx/10)
  if spa < 2 then spa = 2 end
  for i=1,#t do
    l = string.sub(t,i,i)
    drawLetter(l,x+xx,y,sx,sy,c)
    xx = xx + _f(sx + spa)
  end
end

function drawLetter(l,x,y,sx,sy,c)
  if letter[l] then
    local s1 = level.scale*sx
    local s2 = level.scale*sy
    x = x * level.scale
    y = y * level.scale

    --gr.setLineWidth(1+(1+_s(time*2))*level.scale)
    gr.setColor(unpack(c))
    for a=1,#letter[l] do
      gr.line(x+(letter[l][a][1][1]*s1+.5), y+(letter[l][a][1][2]*s2+.5), x+(letter[l][a][2][1]*s1+.5), y+(letter[l][a][2][2]*s2+.5))
    end
  end
end

function genocide()
  local m = 1
  for i, e in pairs(enemy) do
    e.dead = true
    spawnExplosion(e.x,e.y)
    player.score = player.score + (e.points * player.multiplier) * m
    player.kills = player.kills + 1
    m = m * 1.5
  end
end

function loadEnemies(x)
  for y=0,15-1 do
    if level.map[x][y].enemy > 0 then
      --print("Enemy at "..y)
      enemy[#enemy+1] = { x=x*32, y=y*32, points = 100, type = level.map[x][y].enemy, tsb = 0, dead = false }
    end
  end
end

function love.mousepressed(x, y, button)
  if button == "l" then

  end
end

function love.keypressed(k)
  if k == "p" then
    if paused then paused = false else paused = true end
  elseif k == "s" then
    giveSheild()
  elseif k == "g" then
    genocide()
  end
end

function spawnExplosion(x,y)
  explosion[#explosion+1] = { x = x, y = y, life = 32, dead = false }
end

function updateExplosions(dt)
  for i, e in pairs(explosion) do
    e.life = e.life - 64 * dt
    if e.life < 0 then e.dead = true end
  end
end

function drawExplosions()
  for i, e in pairs(explosion) do
    if e.dead == false then
      gr.setColor(255,0,0,255-(3*e.life))
      gr.circle("fill", (e.x-level.offset)*level.scale, e.y*level.scale, (32-e.life)*level.scale, 32)
    end
  end
end

function spawnBullet(x,y,t,a,s,w)
  if t == 1 then
    l = 1
  elseif t == 2 then
    l = 1
  elseif t == 3 then
    l = 5
  else
    l = 100
  end
  bullet[#bullet+1] = { x = x, y = y, type = t, speed = s, angle = a, life = l, who = w, dead = false }
end

function degToRad(d) return d * pi / 180 end

function sort(T) table.sort(T, function(a, b) return a.y < b.y end ) end

function formatNumber(number) return (string.format("%d", number):reverse():gsub( "(%d%d%d)" , "%1," ):reverse():gsub("^(-?),","%1")) end

function overlap(box1x, box1y, box1w, box1h, box2x, box2y, box2w, box2h) if box1x > box2x + box2w - 1 or box1y > box2y + box2h - 1 or box2x > box1x + box1w - 1 or box2y > box1y + box1h - 1 then return false else return true end end

function playSound(s)
  soundmanager:play(sfx[s])
end
